@@ -6,12 +6,14 @@
  */
 
 #include "SettingsBase.h"
+#include "../core/kcmmetadatahelpers.h"
 #include "BaseConfig.h"
 #include "systemsettings_app_debug.h"
 
 #include <QDir>
 #include <QFileInfo>
 #include <QFontDatabase>
+#include <QGuiApplication>
 #include <QLoggingCategory>
 #include <QMenu>
 #include <QMenuBar>
@@ -26,6 +28,7 @@
 #include <KConfigGroup>
 #include <KDesktopFile>
 #include <KFileUtils>
+#include <KLocalizedString>
 #include <KMessageBox>
 #include <KPluginMetaData>
 #include <KServiceTypeTrader>
@@ -44,7 +47,6 @@ SettingsBase::SettingsBase(BaseMode::ApplicationMode mode, QWidget *parent)
     // Ensure delayed loading doesn't cause a crash
     activeView = nullptr;
     aboutDialog = nullptr;
-    configDialog = nullptr;
     lostFound = nullptr;
     // Prepare the view area
     stackedWidget = new QStackedWidget(this);
@@ -58,8 +60,6 @@ SettingsBase::SettingsBase(BaseMode::ApplicationMode mode, QWidget *parent)
     setProperty("_breeze_no_separator", true);
 
     if (m_mode == BaseMode::InfoCenter) {
-        actionCollection()->removeAction(configureAction);
-        configureAction = nullptr;
         setWindowTitle(i18n("Info Center"));
         setWindowIcon(QIcon::fromTheme(QStringLiteral("hwinfo")));
     } else {
@@ -94,12 +94,9 @@ QSize SettingsBase::sizeHint() const
 void SettingsBase::initApplication()
 {
     // Prepare the menu of all modules
-    if (m_mode == BaseMode::InfoCenter) {
-        modules = KServiceTypeTrader::self()->query(QStringLiteral("KCModule"), QStringLiteral("[X-KDE-ParentApp] == 'kinfocenter'"));
-    } else {
-        modules = KServiceTypeTrader::self()->query(QStringLiteral("KCModule"), QStringLiteral("[X-KDE-System-Settings-Parent-Category] != ''"));
-        modules += KServiceTypeTrader::self()->query(QStringLiteral("SystemSettingsExternalApp"));
-    }
+    auto source = m_mode == BaseMode::InfoCenter ? MetaDataSource::KInfoCenter : MetaDataSource::SystemSettings;
+    pluginModules = findKCMsMetaData(source) << findExternalKCMModules(source);
+
     const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::AppDataLocation, QStringLiteral("categories"), QStandardPaths::LocateDirectory);
     categories = KFileUtils::findAllUniqueFiles(dirs, QStringList(QStringLiteral("*.desktop")));
 
@@ -108,11 +105,10 @@ void SettingsBase::initApplication()
 
     // Handle lost+found modules...
     if (lostFound) {
-        for (int i = 0; i < modules.size(); ++i) {
-            const KService::Ptr entry = modules.at(i);
-            MenuItem *infoItem = new MenuItem(false, lostFound);
-            infoItem->setService(entry);
-            qCDebug(SYSTEMSETTINGS_APP_LOG) << "Added " << entry->name();
+        for (const auto &metaData : qAsConst(pluginModules)) {
+            auto infoItem = new MenuItem(false, lostFound);
+            infoItem->setMetaData(metaData);
+            qCDebug(SYSTEMSETTINGS_APP_LOG) << "Added " << metaData.pluginId();
         }
     }
 
@@ -120,22 +116,16 @@ void SettingsBase::initApplication()
     BaseData::instance()->setMenuItem(rootModule);
     BaseData::instance()->setHomeItem(homeModule);
     // Load all possible views
-    const QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("systemsettingsview/"));
+    const QVector<KPluginMetaData> plugins = KPluginMetaData::findPlugins(QStringLiteral("systemsettingsview/"));
 
     for (const KPluginMetaData &plugin : plugins) {
-        KPluginLoader loader(plugin.fileName());
-        KPluginFactory *factory = loader.factory();
-        if (!factory) {
-            qCWarning(SYSTEMSETTINGS_APP_LOG) << "KPluginFactory could not load the plugin:" << plugin.pluginId() << loader.errorString();
-            continue;
-        }
-
-        BaseMode *controller = factory->create<BaseMode>(this, {m_mode, m_startupModule, m_startupModuleArgs});
-        if (!controller) {
-            qCWarning(SYSTEMSETTINGS_APP_LOG) << "Error loading plugin";
+        auto controllerResult = KPluginFactory::instantiatePlugin<BaseMode>(plugin, this, {m_mode, m_startupModule, m_startupModuleArgs});
+        if (!controllerResult) {
+            qCWarning(SYSTEMSETTINGS_APP_LOG) << "Error loading plugin" << controllerResult.errorText;
             continue;
         }
 
+        auto controller = controllerResult.plugin;
         possibleViews.insert(plugin.pluginId(), controller);
         controller->init(plugin);
         connect(controller, &BaseMode::changeToolBarItems, this, &SettingsBase::changeToolBar);
@@ -159,17 +149,24 @@ void SettingsBase::initToolBar()
     // Exit is the very last action
     quitAction = actionCollection()->addAction(KStandardAction::Quit, QStringLiteral("quit_action"), this, SLOT(close()));
 
-    // Configure goes at the end
-    configureAction = actionCollection()->addAction(KStandardAction::Preferences, QStringLiteral("configure"), this, SLOT(configShow()));
-    configureAction->setText(i18n("Configure…"));
+    if (m_mode == BaseMode::SystemSettings) {
+        switchToIconAction = actionCollection()->addAction(QStringLiteral("switchto_iconview"), this, [this] {
+            BaseConfig::setActiveView(QStringLiteral("systemsettings_icon_mode"));
+            changePlugin();
+        });
+        switchToIconAction->setText(i18n("Switch to Icon View"));
+        switchToIconAction->setIcon(QIcon::fromTheme(QStringLiteral("view-list-icons")));
+
+        switchToSidebarAction = actionCollection()->addAction(QStringLiteral("switchto_sidebar"), this, [this] {
+            BaseConfig::setActiveView(QStringLiteral("systemsettings_sidebar_mode"));
+            changePlugin();
+        });
+        switchToSidebarAction->setText(i18n("Switch to Sidebar View"));
+        switchToSidebarAction->setIcon(QIcon::fromTheme(QStringLiteral("view-sidetree")));
+    }
+
     // Help after it
     initHelpMenu();
-    configureAction->setIcon(QIcon::fromTheme(QStringLiteral("settings-configure")));
-
-    // There's nothing to configure in info center mode
-    if (m_mode == BaseMode::InfoCenter) {
-        configureAction->setVisible(false);
-    }
 
     // Then a spacer so the search line-edit is kept separate
     spacerAction = new QWidgetAction(this);
@@ -199,35 +196,6 @@ void SettingsBase::initHelpMenu()
     aboutViewAction = actionCollection()->addAction(KStandardAction::AboutApp, QStringLiteral("help_about_view"), this, SLOT(about()));
 }
 
-void SettingsBase::initConfig()
-{
-    // Prepare dialog first
-    configDialog = new KConfigDialog(this, QStringLiteral("systemsettingsconfig"), BaseConfig::self());
-    configDialog->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-
-    // Add our page
-    QWidget *configPage = new QWidget(configDialog);
-    configWidget.setupUi(configPage);
-    QString iconName = QApplication::windowIcon().name();
-    configDialog->addPage(configPage, i18nc("General config for System Settings", "General"), iconName);
-    QVBoxLayout *configLayout = new QVBoxLayout;
-    // Get the list of modules
-    foreach (BaseMode *mode, possibleViews) {
-        mode->addConfiguration(configDialog);
-        QRadioButton *radioButton = new QRadioButton(mode->metaData().name(), configWidget.GbViewStyle);
-        radioButton->setIcon(QIcon::fromTheme(mode->metaData().iconName()));
-        configLayout->addWidget(radioButton);
-        viewSelection.addButton(radioButton, possibleViews.values().indexOf(mode));
-    }
-    configWidget.GbViewStyle->setLayout(configLayout);
-    configWidget.GbViewStyle->setVisible(possibleViews.count() > 1);
-    KWindowConfig::restoreWindowSize(configDialog->windowHandle(), KSharedConfig::openConfig()->group("ConfigDialog"));
-    connect(configDialog, &KConfigDialog::accepted, this, &SettingsBase::configUpdated);
-    // Need to set size manually here since the .ui file doesn't produce the right
-    // autogenerated code to do it correctly
-    configDialog->resize(QSize(420, 300));
-}
-
 void SettingsBase::initMenuList(MenuItem *parent)
 {
     // look for any categories inside this level, and recurse into them
@@ -256,92 +224,43 @@ void SettingsBase::initMenuList(MenuItem *parent)
         }
     }
 
-    KService::List removeList;
-
     // scan for any modules at this level and add them
-    for (int i = 0; i < modules.size(); ++i) {
-        const KService::Ptr entry = modules.at(i);
-
+    for (const auto &metaData : qAsConst(pluginModules)) {
         QString category;
-        QString category2;
+        QString categoryv2;
         if (m_mode == BaseMode::InfoCenter) {
-            category = entry->property(QStringLiteral("X-KDE-KInfoCenter-Category")).toString();
+            category = metaData.value(QStringLiteral("X-KDE-KInfoCenter-Category"));
         } else {
-            category = entry->property(QStringLiteral("X-KDE-System-Settings-Parent-Category")).toString();
-            category2 = entry->property(QStringLiteral("X-KDE-System-Settings-Parent-Category-V2")).toString();
+            category = metaData.value(QStringLiteral("X-KDE-System-Settings-Parent-Category"));
+            categoryv2 = metaData.value(QStringLiteral("X-KDE-System-Settings-Parent-Category-V2"));
         }
-
         const QString parentCategoryKcm = parent->systemsettingsCategoryModule();
         bool isCategoryOwner = false;
 
-        if (!parentCategoryKcm.isEmpty() && parentCategoryKcm == entry->library()) {
-            parent->setItem(KCModuleInfo(entry));
+        if (!parentCategoryKcm.isEmpty() && parentCategoryKcm == metaData.pluginId()) {
+            parent->setMetaData(metaData);
             isCategoryOwner = true;
         }
 
-        if (!parent->category().isEmpty() && (category == parent->category() || category2 == parent->category())) {
-            if (!entry->noDisplay()) {
+        if (!parent->category().isEmpty() && (category == parent->category() || categoryv2 == parent->category())) {
+            if (!metaData.isHidden()) {
                 // Add the module info to the menu
                 MenuItem *infoItem = new MenuItem(false, parent);
-                infoItem->setService(entry);
+                infoItem->setMetaData(metaData);
                 infoItem->setCategoryOwner(isCategoryOwner);
 
-                if (m_mode == BaseMode::InfoCenter && entry->pluginKeyword() == QStringLiteral("kcm-about-distro")) {
+                if (m_mode == BaseMode::InfoCenter && metaData.pluginId() == QStringLiteral("kcm_about-distro")) {
                     homeModule = infoItem;
-                } else if (m_mode == BaseMode::SystemSettings && entry->pluginKeyword() == QStringLiteral("kcm_landingpage")) {
+                } else if (m_mode == BaseMode::SystemSettings && metaData.pluginId() == QStringLiteral("kcm_landingpage")) {
                     homeModule = infoItem;
                 }
             }
-
-            removeList.append(modules.at(i));
         }
     }
 
-    for (int i = 0; i < removeList.size(); ++i) {
-        modules.removeOne(removeList.at(i));
-    }
-
     parent->sortChildrenByWeight();
 }
 
-void SettingsBase::configUpdated()
-{
-    KConfigGroup dialogConfig = KSharedConfig::openConfig()->group("ConfigDialog");
-    KWindowConfig::saveWindowSize(configDialog->windowHandle(), dialogConfig);
-    BaseConfig::setActiveView(possibleViews.keys().at(viewSelection.checkedId()));
-
-    BaseConfig::setShowToolTips(configWidget.ChTooltips->isChecked());
-    activeView->setShowToolTips(configWidget.ChTooltips->isChecked());
-    activeView->saveConfiguration();
-    changePlugin();
-}
-
-void SettingsBase::configShow()
-{
-    // Initialise the configuration dialog if it hasn't already
-    if (!configDialog) {
-        initConfig();
-    }
-    if (activeView && activeView->moduleView() && !activeView->moduleView()->resolveChanges()) {
-        return; // It shouldn't be triggering anyway, since the action is disabled
-    }
-    if (activeView) {
-        activeView->loadConfiguration();
-    }
-
-    const QStringList pluginList = possibleViews.keys();
-    const int configIndex = pluginList.indexOf(BaseConfig::activeView());
-    if (configIndex != -1) {
-        viewSelection.button(configIndex)->setChecked(true);
-    }
-    configWidget.ChTooltips->setChecked(BaseConfig::showToolTips());
-    if (pluginList.isEmpty()) {
-        KMessageBox::error(this, i18n("System Settings was unable to find any views, and hence nothing is available to configure."), i18n("No views found"));
-    } else {
-        configDialog->show();
-    }
-}
-
 bool SettingsBase::queryClose()
 {
     bool changes = true;
@@ -423,13 +342,10 @@ void SettingsBase::changePlugin()
     // Handle the tooltips
     qDeleteAll(tooltipManagers);
     tooltipManagers.clear();
-    if (BaseConfig::showToolTips()) {
-        QList<QAbstractItemView *> theViews = activeView->views();
-        foreach (QAbstractItemView *view, theViews) {
-            tooltipManagers << new ToolTipManager(view);
-        }
+    const QList<QAbstractItemView *> theViews = activeView->views();
+    for (QAbstractItemView *view : theViews) {
+        tooltipManagers << new ToolTipManager(view);
     }
-    activeView->setShowToolTips(BaseConfig::showToolTips());
 
     changeAboutMenu(activeView->aboutData(), aboutViewAction, i18n("About Active View"));
     viewChange(false);
@@ -442,15 +358,7 @@ void SettingsBase::changePlugin()
 
 void SettingsBase::viewChange(bool state)
 {
-    KCModuleInfo *moduleInfo = activeView->moduleView()->activeModule();
-    if (configureAction) {
-        configureAction->setDisabled(state);
-    }
-    if (moduleInfo) {
-        setCaption(moduleInfo->moduleName(), state);
-    } else {
-        setCaption(QString(), state);
-    }
+    setCaption(activeView->moduleView()->activeModuleName(), state);
 }
 
 void SettingsBase::updateViewActions()
@@ -473,9 +381,9 @@ void SettingsBase::changeToolBar(BaseMode::ToolBarItems toolbar)
         guiFactory()->plugActionList(this, QStringLiteral("search"), searchBarActions);
         actionCollection()->setDefaultShortcut(searchAction, QKeySequence(Qt::CTRL | Qt::Key_F));
     }
-    if ((BaseMode::Configure & toolbar) && configureAction) {
+    if ((BaseMode::Configure & toolbar) && switchToSidebarAction) {
         QList<QAction *> configureBarActions;
-        configureBarActions << configureAction;
+        configureBarActions << switchToSidebarAction;
         guiFactory()->plugActionList(this, QStringLiteral("configure"), configureBarActions);
     }
     if (BaseMode::Quit & toolbar) {
@@ -501,11 +409,11 @@ void SettingsBase::changeAboutMenu(const KAboutData *menuAbout, QAction *menuIte
 
     if (menuAbout) {
         menuItem->setText(i18n("About %1", menuAbout->displayName()));
-        menuItem->setIcon(QIcon::fromTheme(menuAbout->programIconName()));
+        menuItem->setIcon(QGuiApplication::windowIcon());
         menuItem->setEnabled(true);
     } else {
         menuItem->setText(fallback);
-        menuItem->setIcon(QIcon::fromTheme(KAboutData::applicationData().programIconName()));
+        menuItem->setIcon(QGuiApplication::windowIcon());
         menuItem->setEnabled(false);
     }
 }
