@@ -1,6 +1,7 @@
 /*
  *   SPDX-FileCopyrightText: 2009 Ben Cooksley <bcooksley@kde.org>
  *   SPDX-FileCopyrightText: 2009 Mathias Soeken <msoeken@informatik.uni-bremen.de>
+ *   SPDX-FileCopyrightText: 2021 Harald Sitter <sitter@kde.org>
  *
  *   SPDX-License-Identifier: GPL-2.0-or-later
  */
@@ -38,6 +39,7 @@
 #include <KActivities/ResourceInstance>
 
 #include <cmath>
+#include <kpluginmetadata.h>
 
 #include "MenuItem.h"
 
@@ -111,7 +113,7 @@ public:
     {
     }
     QMap<KPageWidgetItem *, KCModuleProxy *> mPages;
-    QMap<KPageWidgetItem *, KCModuleInfo *> mModules;
+    QMap<KPageWidgetItem *, QString> mPagesPluginIdMap;
     KPageWidget *mPageWidget = nullptr;
     CustomTitle *mCustomHeader = nullptr;
     QVBoxLayout *mLayout = nullptr;
@@ -193,9 +195,9 @@ ModuleView::~ModuleView()
     delete d;
 }
 
-KCModuleInfo *ModuleView::activeModule() const
+QString ModuleView::activeModuleName() const
 {
-    return d->mModules.value(d->mPageWidget->currentPage());
+    return d->mPageWidget->currentPage() ? d->mPageWidget->currentPage()->name() : QString();
 }
 
 void ModuleView::loadModule(const QModelIndex &menuItem, const QStringList &args)
@@ -207,61 +209,54 @@ void ModuleView::loadModule(const QModelIndex &menuItem, const QStringList &args
     MenuItem *item = menuItem.data(Qt::UserRole).value<MenuItem *>();
 
     // if module has a main page (like in Appearance > Global Theme) we'll load that
-    if (!item->item().library().isEmpty() || item->isExternalAppModule()) {
-        addModule(&item->item(), args);
+    if (item->isLibrary() || item->isExternalAppModule()) {
+        addModule(item, args);
     }
     // if module doesn't have a main page, we'll load the first subpage
     else if (menuItem.model()->rowCount(menuItem) > 0) {
         MenuItem *subpageItem = menuItem.model()->index(0, 0, menuItem).data(Qt::UserRole).value<MenuItem *>();
-        addModule(&subpageItem->item(), args);
+        addModule(subpageItem, args);
     }
 }
 
-void ModuleView::addModule(KCModuleInfo *module, const QStringList &args)
+void ModuleView::addModule(MenuItem *item, const QStringList &args)
 {
-    if (!module || !module->service()->isValid()) {
-        return;
-    }
-    if (!module->service()) {
-        qWarning() << "ModuleInfo has no associated KService";
-        return;
-    }
-    if (!KAuthorized::authorizeControlModule(module->service()->menuId())) {
+    const KPluginMetaData data = item->metaData();
+    if (!KAuthorized::authorizeControlModule(data.pluginId())) {
         qWarning() << "Not authorised to load module";
         return;
     }
-    if (module->service()->noDisplay()) {
+    if (data.isHidden()) {
         return;
     }
 
-    if (KPageWidgetItem *page = d->mModules.key(module)) {
+    if (KPageWidgetItem *page = d->mPagesPluginIdMap.key(data.name())) {
         activeModuleChanged(page, d->mPageWidget->currentPage());
         return;
     }
 
     // Create the scroller
-    QScrollArea *moduleScroll = new QScrollArea(this);
+    auto *moduleScroll = new QScrollArea(this);
     // Prepare the scroll area
     moduleScroll->setWidgetResizable(true);
     moduleScroll->setFrameStyle(QFrame::NoFrame);
     moduleScroll->viewport()->setAutoFillBackground(false);
     // Create the page
-    KPageWidgetItem *page = new KPageWidgetItem(moduleScroll, module->moduleName());
+    auto *page = new KPageWidgetItem(moduleScroll, data.name());
     // Provide information to the users
 
-    if (module->service()->hasServiceType(QStringLiteral("SystemSettingsExternalApp")) || // Is it an external app?
-        module->service()->substituteUid()) { // ...or does it require UID substitution?
-        QWidget *externalWidget = new ExternalAppModule(this, module->service());
+    if (item->isExternalAppModule()) {
+        auto *externalWidget = new ExternalAppModule(this, KService::Ptr(new KService(item->metaData().metaDataFileName())));
         moduleScroll->setWidget(externalWidget);
     } else { // It must be a normal module then
-        KCModuleProxy *moduleProxy = new KCModuleProxy(*module, moduleScroll, args);
+        auto *moduleProxy = new KCModuleProxy(data, moduleScroll, args);
         moduleScroll->setWidget(moduleProxy);
         moduleProxy->setAutoFillBackground(false);
-        connect(moduleProxy, SIGNAL(changed(bool)), this, SLOT(stateChanged()));
+        connect(moduleProxy, &KCModuleProxy::changed, this, &ModuleView::stateChanged);
         d->mPages.insert(page, moduleProxy);
     }
 
-    d->mModules.insert(page, module);
+    d->mPagesPluginIdMap.insert(page, data.name());
     updatePageIconHeader(page, true);
     // Add the new page
     d->mPageWidget->addPage(page);
@@ -275,18 +270,18 @@ void ModuleView::updatePageIconHeader(KPageWidgetItem *page, bool light)
     }
 
     KCModuleProxy *moduleProxy = d->mPages.value(page);
-    KCModuleInfo *moduleInfo = d->mModules.value(page);
 
-    if (!moduleInfo) {
+    if (!moduleProxy || !moduleProxy->metaData().isValid()) {
         // Seems like we have some form of a race condition going on here...
         return;
     }
 
-    page->setHeader(moduleInfo->moduleName());
-    page->setIcon(QIcon::fromTheme(moduleInfo->icon()));
+    const QString currentModuleName = moduleProxy->metaData().name();
+    page->setHeader(currentModuleName);
+    page->setIcon(QIcon::fromTheme(moduleProxy->metaData().iconName()));
 
-    const bool isQml = (moduleProxy && moduleProxy->realModule() && moduleProxy->realModule()->inherits("KCModuleQml"));
-    if (activeModule() == moduleInfo) {
+    const bool isQml = (moduleProxy->realModule() && moduleProxy->realModule()->inherits("KCModuleQml"));
+    if (d->mPageWidget->currentPage() && d->mPageWidget->currentPage()->name() == currentModuleName) {
         d->mCustomHeader->setVisible(!isQml);
     }
     page->setHeaderVisible(false);
@@ -294,12 +289,6 @@ void ModuleView::updatePageIconHeader(KPageWidgetItem *page, bool light)
     if (light) {
         return;
     }
-
-    if (moduleProxy && moduleProxy->realModule()->useRootOnlyMessage()) {
-        page->setHeader(moduleInfo->moduleName() + QStringLiteral("<br><small>") + moduleProxy->realModule()->rootOnlyMessage() + QStringLiteral("</small>"));
-        d->mCustomHeader->setText(moduleInfo->moduleName() + QStringLiteral("<br><small>") + moduleProxy->realModule()->rootOnlyMessage()
-                                  + QStringLiteral("</small>"));
-    }
 }
 
 bool ModuleView::resolveChanges()
@@ -310,7 +299,7 @@ bool ModuleView::resolveChanges()
 
 bool ModuleView::resolveChanges(KCModuleProxy *currentProxy)
 {
-    if (!currentProxy || !currentProxy->changed()) {
+    if (!currentProxy || !currentProxy->isChanged()) {
         return true;
     }
 
@@ -341,14 +330,12 @@ void ModuleView::closeModules()
 {
     d->pageChangeSupressed = true;
     d->mApplyAuthorize->setAuthAction(KAuth::Action()); // Ensure KAuth knows that authentication is now pointless...
-    QMap<KPageWidgetItem *, KCModuleInfo *>::iterator page = d->mModules.begin();
-    QMap<KPageWidgetItem *, KCModuleInfo *>::iterator pageEnd = d->mModules.end();
-    for (; page != pageEnd; ++page) {
+    for (auto page = d->mPagesPluginIdMap.cbegin(); page != d->mPagesPluginIdMap.cend(); ++page) {
         d->mPageWidget->removePage(page.key());
     }
 
     d->mPages.clear();
-    d->mModules.clear();
+    d->mPagesPluginIdMap.clear();
     d->pageChangeSupressed = false;
 }
 
@@ -387,12 +374,12 @@ void ModuleView::moduleDefaults()
 
 void ModuleView::moduleHelp()
 {
-    KCModuleInfo *activeModule = d->mModules.value(d->mPageWidget->currentPage());
+    KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
     if (!activeModule) {
         return;
     }
 
-    QString docPath = activeModule->docPath();
+    const QString docPath = activeModule->metaData().value(QStringLiteral("X-DocPath"));
     if (docPath.isEmpty()) {
         return;
     }
@@ -417,17 +404,16 @@ void ModuleView::activeModuleChanged(KPageWidgetItem *current, KPageWidgetItem *
     // We need to get the state of the now active module
     stateChanged();
 
-    KCModuleInfo *activeModuleInfo = activeModule();
     KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
-    if (activeModule || activeModuleInfo) {
+    if (activeModule) {
         // TODO: if we'll ever need statistics for kinfocenter modules, save them with an URL like "kinfo:"
 
-        if (activeModule && d->mSaveStatistics && activeModule->moduleInfo().service()->desktopEntryName() != QStringLiteral("kcm_landingpage")) {
-            KActivities::ResourceInstance::notifyAccessed(QUrl(QStringLiteral("kcm:") + activeModule->moduleInfo().service()->storageId()),
+        if (d->mSaveStatistics && activeModule->metaData().pluginId() != QStringLiteral("kcm_landingpage")) {
+            KActivities::ResourceInstance::notifyAccessed(QUrl(QStringLiteral("kcm:") + activeModule->metaData().pluginId()),
                                                           QStringLiteral("org.kde.systemsettings"));
         }
 
-        d->mCustomHeader->setText(activeModuleInfo->moduleName());
+        d->mCustomHeader->setText(activeModule->metaData().name());
 
         d->mLayout->setContentsMargins(0, 0, 0, 0);
         d->mLayout->setSpacing(0);
@@ -477,7 +463,7 @@ void ModuleView::stateChanged()
     KCModule::Buttons buttons = KCModule::NoAdditionalButton;
     if (activeModule) {
         buttons = activeModule->buttons();
-        change = activeModule->changed();
+        change = activeModule->isChanged();
         defaulted = activeModule->defaulted();
 
         disconnect(d->mApplyAuthorize, SIGNAL(authorized(KAuth::Action)), this, SLOT(moduleSave()));
