@@ -17,6 +17,8 @@
 class KCModuleInfo;
 class KCModuleProxy;
 class KPageWidgetItem;
+class KPluginMetaData;
+class MenuItem;
 
 /**
  * @brief Provides a convenient way to display modules
@@ -55,7 +57,7 @@ public:
      * Provides the module information, which is used to set the caption of the window when either the
      * active module or configuration changes.
      */
-    KCModuleInfo *activeModule() const;
+    QString activeModuleName() const;
 
     /**
      * Resolves any changes in the currently active module by prompting the user if they exist.
@@ -163,7 +165,7 @@ public Q_SLOTS:
 
 private:
     bool resolveChanges(KCModuleProxy *currentProxy);
-    void addModule(KCModuleInfo *module, const QStringList &args);
+    void addModule(MenuItem *item, const QStringList &args);
     bool moduleSave(KCModuleProxy *module);
     void updatePageIconHeader(KPageWidgetItem *page, bool light = false);
 
@@ -189,7 +191,7 @@ Q_SIGNALS:
     void closeRequest();
 
     /**
-     * Emmitted when showDefaultsIndicators state changed
+     * Emitted when showDefaultsIndicators state changed
      */
     void showDefaultsIndicatorsChanged(bool show);
 
