@@ -12,14 +12,15 @@
 #include "MenuProxyModel.h"
 #include "ModuleView.h"
 #include "ToolTips/tooltipmanager.h"
+#include "kcmmetadatahelpers.h"
 
 #include <QGuiApplication>
 #include <QHBoxLayout>
 
 #include <KAboutData>
 #include <KActionCollection>
-#include <KCModuleInfo>
 #include <KCMUtils/KCModuleLoader>
+#include <KCModuleInfo>
 #include <KConfigGroup>
 #include <KDescendantsProxyModel>
 #include <KLocalizedContext>
@@ -36,6 +37,7 @@
 #include <QQmlEngine>
 #include <QQuickWidget>
 #include <QStandardItemModel>
+#include <QStandardPaths>
 
 #include <KActivities/Stats/ResultModel>
 #include <KActivities/Stats/ResultSet>
@@ -110,7 +112,7 @@ void SubcategoryModel::setParentIndex(const QModelIndex &activeModule)
 void SubcategoryModel::loadParentCategoryModule()
 {
     MenuItem *menuItem = m_activeModuleIndex.data(MenuModel::MenuItemRole).value<MenuItem *>();
-    if (!menuItem->item().library().isEmpty()) {
+    if (menuItem->isLibrary()) {
         m_sidebarMode->loadModule(m_activeModuleIndex);
     }
 }
@@ -196,7 +198,8 @@ public:
                 m_resultModel->forgetResource(QStringLiteral("kcm:") % desktopName);
                 return QVariant();
             }
-            mi->setService(service);
+            QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("kservices5/") + service->entryPath());
+            mi->setMetaData(KPluginMetaData::fromDesktopFile(path));
         }
 
         switch (role) {
@@ -287,8 +290,8 @@ SidebarMode::SidebarMode(QObject *parent, const QVariantList &args)
     d->aboutIcon->addAuthor(i18n("Ben Cooksley"), i18n("Author"), QStringLiteral("bcooksley@kde.org"));
     d->aboutIcon->addAuthor(i18n("Mathias Soeken"), i18n("Developer"), QStringLiteral("msoeken@informatik.uni-bremen.de"));
 
-    qmlRegisterType<QAction>();
-    qmlRegisterType<QAbstractItemModel>();
+    qmlRegisterAnonymousType<QAction>("", 1);
+    qmlRegisterAnonymousType<QAbstractItemModel>("", 1);
 }
 
 SidebarMode::~SidebarMode()
@@ -408,7 +411,7 @@ QString SidebarMode::actionIconName(const QString &name) const
 
 void SidebarMode::requestToolTip(const QModelIndex &index, const QRectF &rect)
 {
-    if (showToolTips() && index.model()) {
+    if (index.model()) {
         d->toolTipManager->setModel(index.model());
         d->toolTipManager->requestToolTip(index, rect.toRect());
     }
@@ -416,9 +419,7 @@ void SidebarMode::requestToolTip(const QModelIndex &index, const QRectF &rect)
 
 void SidebarMode::requestMostUsedToolTip(int index, const QRectF &rect)
 {
-    if (showToolTips()) {
-        d->mostUsedToolTipManager->requestToolTip(d->mostUsedModel->index(index, 0), rect.toRect());
-    }
+    d->mostUsedToolTipManager->requestToolTip(d->mostUsedModel->index(index, 0), rect.toRect());
 }
 
 void SidebarMode::hideToolTip()
@@ -441,11 +442,13 @@ void SidebarMode::showActionMenu(const QPoint &position)
     });
     menu->setAttribute(Qt::WA_DeleteOnClose);
 
-    const QStringList actionList{QStringLiteral("configure"),
+    const QStringList actionList{QStringLiteral("switchto_iconview"),
+                                 QStringLiteral("configure"),
                                  QStringLiteral("help_contents"),
                                  QStringLiteral("help_report_bug"),
                                  QStringLiteral("help_about_app"),
                                  QStringLiteral("help_about_kde")};
+
     for (const QString &actionName : actionList) {
         menu->addAction(d->collection->action(actionName));
     }
@@ -467,7 +470,7 @@ void SidebarMode::loadModule(const QModelIndex &activeModule, const QStringList
     }
 
     // If we are trying to load a module already open
-    if (d->moduleView->activeModule() && mi->item() == *d->moduleView->activeModule()) {
+    if (mi->name() == d->moduleView->activeModuleName()) {
         return;
     }
 
@@ -547,7 +550,7 @@ void SidebarMode::loadModule(const QModelIndex &activeModule, const QStringList
             QModelIndex idx = d->flatModel->index(i, 0);
             MenuItem *otherMi = idx.data(MenuModel::MenuItemRole).value<MenuItem *>();
 
-            if (otherMi->item() == mi->item()) {
+            if (mi->metaData().isValid() && otherMi->metaData() == mi->metaData()) {
                 flatIndex = idx;
                 break;
             }
@@ -611,7 +614,7 @@ void SidebarMode::updateDefaults()
         return;
     }
 
-    auto *moduleData = KCModuleLoader::loadModuleData(item->item());
+    KCModuleData *moduleData = loadModuleData(item->metaData());
     if (moduleData) {
         connect(moduleData, &KCModuleData::loaded, this, [this, item, moduleData, categoryIdx]() {
             item->setDefaultIndicator(!moduleData->isDefaults());
@@ -708,7 +711,7 @@ void SidebarMode::refreshDefaults()
                 continue;
             }
 
-            auto *moduleData = KCModuleLoader::loadModuleData(item->item());
+            KCModuleData *moduleData = loadModuleData(item->metaData());
             if (moduleData) {
                 connect(moduleData, &KCModuleData::loaded, this, [this, item, moduleData]() {
                     item->setDefaultIndicator(!moduleData->isDefaults());
@@ -872,7 +875,7 @@ void SidebarMode::reloadStartupModule()
 
 bool SidebarMode::eventFilter(QObject *watched, QEvent *event)
 {
-    // FIXME: those are all workarounds around the QQuickWidget brokeness
+    // FIXME: those are all workarounds around the QQuickWidget brokenness
     if ((watched == d->quickWidget || watched == d->placeHolderWidget) && event->type() == QEvent::KeyPress) {
         // allow tab navigation inside the qquickwidget
         QKeyEvent *ke = static_cast<QKeyEvent *>(event);
